local LibStub = LibStub
local AceAddon = LibStub("AceAddon-3.0")
local AceLocale = LibStub("AceLocale-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local ControlPanel = AceAddon:GetAddon("ControlPanel")
local ControlPanel_Options = AceAddon:NewAddon("ControlPanel_Options")
local L = AceLocale:GetLocale("ControlPanel_Options")
local LSM = LibStub("LibSharedMedia-3.0")
local string_format = string.format
local string_upper = string.upper
local SetCVar = SetCVar
local GetCVar = GetCVar
local tonumber = tonumber
local GetCVarBool = GetCVarBool
local GetCVarDefault = GetCVarDefault
local GetClassInfoByID = GetClassInfoByID
local ShowCloak = ShowCloak
local RestartGx = RestartGx
local Sound_GameSystem_RestartSoundSystem = Sound_GameSystem_RestartSoundSystem
local Sound_GameSystem_GetNumOutputDrivers = Sound_GameSystem_GetNumOutputDrivers
local Sound_GameSystem_GetOutputDriverNameByIndex = Sound_GameSystem_GetOutputDriverNameByIndex
--------------------------------------------------------------------------------------

local order = 0
local function get_order()
	local temp = order
	order = order +1
	return temp
end
local function get_cvar_bool(c)
	if GetCVarBool(c) then
		return true
	else
		return false
	end
end
local function get_cvar_number(cvar)
	return tonumber(GetCVar(cvar))
end

local function set_graphics_quality(val)
	if val == 0 then
		ControlPanel:SetCVar("worldbasemip",2)
		ControlPanel:SetCVar("terrainmiplevel",1)
		ControlPanel:SetCVar("farclip",0)
		ControlPanel:SetCVar("ProjectedTextures",false)
		ControlPanel:SetCVar("environmentDetail",50)
		ControlPanel:SetCVar("waterDetail",0)
		ControlPanel:SetCVar("particleDensity",10)
		ControlPanel:SetCVar("groundEffectDensity",16)
		ControlPanel:SetCVar("groundEffectDist",50)
		ControlPanel:SetCVar("textureFilteringMode",0)
		ControlPanel:SetCVar("shadowMode",0)
		ControlPanel:SetCVar("sunShafts",0)
		ControlPanel:SetCVar("SSAO",0)
		ControlPanel:SetCVar("outlineEngineMode",false)
		ControlPanel:SetCVar("DepthBasedOpacity",false)
		ControlPanel:SetCVar("refraction",0)
		ControlPanel:SetCVar("lightmode",0)
	elseif val == 1 then
		ControlPanel:SetCVar("worldbasemip",2)
		ControlPanel:SetCVar("terrainmiplevel",1)
		ControlPanel:SetCVar("farclip",400)
		ControlPanel:SetCVar("ProjectedTextures",false)
		ControlPanel:SetCVar("environmentDetail",50)
		ControlPanel:SetCVar("waterDetail",0)
		ControlPanel:SetCVar("particleDensity",10)
		ControlPanel:SetCVar("groundEffectDensity",16)
		ControlPanel:SetCVar("groundEffectDist",70)
		ControlPanel:SetCVar("textureFilteringMode",1)
		ControlPanel:SetCVar("shadowMode",0)
		ControlPanel:SetCVar("sunShafts",0)
		ControlPanel:SetCVar("SSAO",0)
		ControlPanel:SetCVar("outlineEngineMode",false)
		ControlPanel:SetCVar("DepthBasedOpacity",false)
		ControlPanel:SetCVar("refraction",0)
		ControlPanel:SetCVar("lightmode",0)
	elseif val == 2 then
		ControlPanel:SetCVar("worldbasemip",1)
		ControlPanel:SetCVar("terrainmiplevel",1)
		ControlPanel:SetCVar("farclip",600)
		ControlPanel:SetCVar("ProjectedTextures",false)
		ControlPanel:SetCVar("environmentDetail",75)
		ControlPanel:SetCVar("waterDetail",1)
		ControlPanel:SetCVar("particleDensity",50)
		ControlPanel:SetCVar("groundEffectDensity",40)
		ControlPanel:SetCVar("groundEffectDist",110)
		ControlPanel:SetCVar("textureFilteringMode",2)
		ControlPanel:SetCVar("shadowMode",1)
		ControlPanel:SetCVar("sunShafts",0)
		ControlPanel:SetCVar("SSAO",0)
		ControlPanel:SetCVar("outlineEngineMode",true)
		ControlPanel:SetCVar("DepthBasedOpacity",true)
		ControlPanel:SetCVar("refraction",0)
		ControlPanel:SetCVar("lightmode",1)
	elseif val == 3 then
		ControlPanel:SetCVar("worldbasemip",1)
		ControlPanel:SetCVar("terrainmiplevel",1)
		ControlPanel:SetCVar("farclip",900)
		ControlPanel:SetCVar("ProjectedTextures",true)
		ControlPanel:SetCVar("environmentDetail",100)
		ControlPanel:SetCVar("waterDetail",2)
		ControlPanel:SetCVar("particleDensity",50)
		ControlPanel:SetCVar("groundEffectDensity",64)
		ControlPanel:SetCVar("groundEffectDist",160)
		ControlPanel:SetCVar("textureFilteringMode",3)
		ControlPanel:SetCVar("shadowMode",2)
		ControlPanel:SetCVar("sunShafts",1)
		ControlPanel:SetCVar("SSAO",1)
		ControlPanel:SetCVar("outlineEngineMode",true)
		ControlPanel:SetCVar("DepthBasedOpacity",true)
		ControlPanel:SetCVar("refraction",1)
		ControlPanel:SetCVar("lightmode",2)
	elseif val == 4 then
		ControlPanel:SetCVar("worldbasemip",0)
		ControlPanel:SetCVar("terrainmiplevel",0)
		ControlPanel:SetCVar("farclip",1100)
		ControlPanel:SetCVar("ProjectedTextures",true)
		ControlPanel:SetCVar("environmentDetail",125)
		ControlPanel:SetCVar("waterDetail",2)
		ControlPanel:SetCVar("particleDensity",80)
		ControlPanel:SetCVar("groundEffectDensity",80)
		ControlPanel:SetCVar("groundEffectDist",200)
		ControlPanel:SetCVar("textureFilteringMode",4)
		ControlPanel:SetCVar("shadowMode",3)
		ControlPanel:SetCVar("sunShafts",2)
		ControlPanel:SetCVar("SSAO",2)
		ControlPanel:SetCVar("outlineEngineMode",true)
		ControlPanel:SetCVar("DepthBasedOpacity",true)
		ControlPanel:SetCVar("refraction",2)
		ControlPanel:SetCVar("lightmode",2)
	elseif val == 5 then
		ControlPanel:SetCVar("worldbasemip",0)
		ControlPanel:SetCVar("terrainmiplevel",0)
		ControlPanel:SetCVar("farclip",1300)
		ControlPanel:SetCVar("ProjectedTextures",true)
		ControlPanel:SetCVar("environmentDetail",150)
		ControlPanel:SetCVar("waterDetail",3)
		ControlPanel:SetCVar("particleDensity",80)
		ControlPanel:SetCVar("groundEffectDensity",128)
		ControlPanel:SetCVar("groundEffectDist",260)
		ControlPanel:SetCVar("textureFilteringMode",5)
		ControlPanel:SetCVar("shadowMode",4)
		ControlPanel:SetCVar("sunShafts",2)
		ControlPanel:SetCVar("SSAO",3)
		ControlPanel:SetCVar("outlineEngineMode",true)
		ControlPanel:SetCVar("DepthBasedOpacity",true)
		ControlPanel:SetCVar("refraction",2)
		ControlPanel:SetCVar("lightmode",2)
	elseif val == 6 then
		ControlPanel:SetCVar("worldbasemip",0)
		ControlPanel:SetCVar("terrainmiplevel",0)
		ControlPanel:SetCVar("farclip",1300)
		ControlPanel:SetCVar("ProjectedTextures",true)
		ControlPanel:SetCVar("environmentDetail",150)
		ControlPanel:SetCVar("waterDetail",3)
		ControlPanel:SetCVar("particleDensity",100)
		ControlPanel:SetCVar("groundEffectDensity",128)
		ControlPanel:SetCVar("groundEffectDist",260)
		ControlPanel:SetCVar("textureFilteringMode",5)
		ControlPanel:SetCVar("shadowMode",4)
		ControlPanel:SetCVar("sunShafts",2)
		ControlPanel:SetCVar("SSAO",3)
		ControlPanel:SetCVar("outlineEngineMode",true)
		ControlPanel:SetCVar("DepthBasedOpacity",true)
		ControlPanel:SetCVar("refraction",2)
		ControlPanel:SetCVar("lightmode",2)
	end
end

local function set_raid_graphics_quality(val)
	if val == 0 then
		ControlPanel:SetCVar("RAIDworldbasemip",2)
		ControlPanel:SetCVar("RAIDterrainmiplevel",1)
		ControlPanel:SetCVar("RAIDfarclip",0)
		ControlPanel:SetCVar("RAIDProjectedTextures",false)
		ControlPanel:SetCVar("RAIDenvironmentDetail",50)
		ControlPanel:SetCVar("RAIDwaterDetail",0)
		ControlPanel:SetCVar("RAIDparticleDensity",10)
		ControlPanel:SetCVar("RAIDgroundEffectDensity",16)
		ControlPanel:SetCVar("RAIDgroundEffectDist",50)
		ControlPanel:SetCVar("RAIDtextureFilteringMode",0)
		ControlPanel:SetCVar("RAIDshadowMode",0)
		ControlPanel:SetCVar("RAIDsunShafts",0)
		ControlPanel:SetCVar("RAIDSSAO",0)
		ControlPanel:SetCVar("RAIDoutlineEngineMode",false)
		ControlPanel:SetCVar("RAIDDepthBasedOpacity",false)
		ControlPanel:SetCVar("RAIDrefraction",0)
		ControlPanel:SetCVar("RAIDlightmode",0)
	elseif val == 1 then
		ControlPanel:SetCVar("RAIDworldbasemip",2)
		ControlPanel:SetCVar("RAIDterrainmiplevel",1)
		ControlPanel:SetCVar("RAIDfarclip",400)
		ControlPanel:SetCVar("RAIDProjectedTextures",false)
		ControlPanel:SetCVar("RAIDenvironmentDetail",50)
		ControlPanel:SetCVar("RAIDwaterDetail",0)
		ControlPanel:SetCVar("RAIDparticleDensity",10)
		ControlPanel:SetCVar("RAIDgroundEffectDensity",16)
		ControlPanel:SetCVar("RAIDgroundEffectDist",70)
		ControlPanel:SetCVar("RAIDtextureFilteringMode",1)
		ControlPanel:SetCVar("RAIDshadowMode",0)
		ControlPanel:SetCVar("RAIDsunShafts",0)
		ControlPanel:SetCVar("RAIDSSAO",0)
		ControlPanel:SetCVar("RAIDoutlineEngineMode",false)
		ControlPanel:SetCVar("RAIDDepthBasedOpacity",false)
		ControlPanel:SetCVar("RAIDrefraction",0)
		ControlPanel:SetCVar("RAIDlightmode",0)
	elseif val == 2 then
		ControlPanel:SetCVar("RAIDworldbasemip",1)
		ControlPanel:SetCVar("RAIDterrainmiplevel",1)
		ControlPanel:SetCVar("RAIDfarclip",600)
		ControlPanel:SetCVar("RAIDProjectedTextures",false)
		ControlPanel:SetCVar("RAIDenvironmentDetail",75)
		ControlPanel:SetCVar("RAIDwaterDetail",1)
		ControlPanel:SetCVar("RAIDparticleDensity",50)
		ControlPanel:SetCVar("RAIDgroundEffectDensity",40)
		ControlPanel:SetCVar("RAIDgroundEffectDist",110)
		ControlPanel:SetCVar("RAIDtextureFilteringMode",2)
		ControlPanel:SetCVar("RAIDshadowMode",1)
		ControlPanel:SetCVar("RAIDsunShafts",0)
		ControlPanel:SetCVar("RAIDSSAO",0)
		ControlPanel:SetCVar("RAIDoutlineEngineMode",true)
		ControlPanel:SetCVar("RAIDDepthBasedOpacity",true)
		ControlPanel:SetCVar("RAIDrefraction",0)
		ControlPanel:SetCVar("RAIDlightmode",1)
	elseif val == 3 then
		ControlPanel:SetCVar("RAIDworldbasemip",1)
		ControlPanel:SetCVar("RAIDterrainmiplevel",1)
		ControlPanel:SetCVar("RAIDfarclip",900)
		ControlPanel:SetCVar("RAIDProjectedTextures",true)
		ControlPanel:SetCVar("RAIDenvironmentDetail",100)
		ControlPanel:SetCVar("RAIDwaterDetail",2)
		ControlPanel:SetCVar("RAIDparticleDensity",50)
		ControlPanel:SetCVar("RAIDgroundEffectDensity",64)
		ControlPanel:SetCVar("RAIDgroundEffectDist",160)
		ControlPanel:SetCVar("RAIDtextureFilteringMode",3)
		ControlPanel:SetCVar("RAIDshadowMode",2)
		ControlPanel:SetCVar("RAIDsunShafts",1)
		ControlPanel:SetCVar("RAIDSSAO",1)
		ControlPanel:SetCVar("RAIDoutlineEngineMode",true)
		ControlPanel:SetCVar("RAIDDepthBasedOpacity",true)
		ControlPanel:SetCVar("RAIDrefraction",1)
		ControlPanel:SetCVar("RAIDlightmode",2)
	elseif val == 4 then
		ControlPanel:SetCVar("RAIDworldbasemip",0)
		ControlPanel:SetCVar("RAIDterrainmiplevel",0)
		ControlPanel:SetCVar("RAIDfarclip",1100)
		ControlPanel:SetCVar("RAIDProjectedTextures",true)
		ControlPanel:SetCVar("RAIDenvironmentDetail",125)
		ControlPanel:SetCVar("RAIDwaterDetail",2)
		ControlPanel:SetCVar("RAIDparticleDensity",80)
		ControlPanel:SetCVar("RAIDgroundEffectDensity",80)
		ControlPanel:SetCVar("RAIDgroundEffectDist",200)
		ControlPanel:SetCVar("RAIDtextureFilteringMode",4)
		ControlPanel:SetCVar("RAIDshadowMode",3)
		ControlPanel:SetCVar("RAIDsunShafts",2)
		ControlPanel:SetCVar("RAIDSSAO",2)
		ControlPanel:SetCVar("RAIDoutlineEngineMode",true)
		ControlPanel:SetCVar("RAIDDepthBasedOpacity",true)
		ControlPanel:SetCVar("RAIDrefraction",2)
		ControlPanel:SetCVar("RAIDlightmode",2)
	elseif val == 5 then
		ControlPanel:SetCVar("RAIDworldbasemip",0)
		ControlPanel:SetCVar("RAIDterrainmiplevel",0)
		ControlPanel:SetCVar("RAIDfarclip",1300)
		ControlPanel:SetCVar("RAIDProjectedTextures",true)
		ControlPanel:SetCVar("RAIDenvironmentDetail",150)
		ControlPanel:SetCVar("RAIDwaterDetail",3)
		ControlPanel:SetCVar("RAIDparticleDensity",80)
		ControlPanel:SetCVar("RAIDgroundEffectDensity",128)
		ControlPanel:SetCVar("RAIDgroundEffectDist",260)
		ControlPanel:SetCVar("RAIDtextureFilteringMode",5)
		ControlPanel:SetCVar("RAIDshadowMode",4)
		ControlPanel:SetCVar("RAIDsunShafts",2)
		ControlPanel:SetCVar("RAIDSSAO",3)
		ControlPanel:SetCVar("RAIDoutlineEngineMode",true)
		ControlPanel:SetCVar("RAIDDepthBasedOpacity",true)
		ControlPanel:SetCVar("RAIDrefraction",2)
		ControlPanel:SetCVar("RAIDlightmode",2)
	elseif val == 6 then
		ControlPanel:SetCVar("RAIDworldbasemip",0)
		ControlPanel:SetCVar("RAIDterrainmiplevel",0)
		ControlPanel:SetCVar("RAIDfarclip",1300)
		ControlPanel:SetCVar("RAIDProjectedTextures",true)
		ControlPanel:SetCVar("RAIDenvironmentDetail",150)
		ControlPanel:SetCVar("RAIDwaterDetail",3)
		ControlPanel:SetCVar("RAIDparticleDensity",100)
		ControlPanel:SetCVar("RAIDgroundEffectDensity",128)
		ControlPanel:SetCVar("RAIDgroundEffectDist",260)
		ControlPanel:SetCVar("RAIDtextureFilteringMode",5)
		ControlPanel:SetCVar("RAIDshadowMode",4)
		ControlPanel:SetCVar("RAIDsunShafts",2)
		ControlPanel:SetCVar("RAIDSSAO",3)
		ControlPanel:SetCVar("RAIDoutlineEngineMode",true)
		ControlPanel:SetCVar("RAIDDepthBasedOpacity",true)
		ControlPanel:SetCVar("RAIDrefraction",2)
		ControlPanel:SetCVar("RAIDlightmode",2)
	end
end

local languages = {deDE="deDE",enUS="enUS",esES="esES",esMX="esMX",frFR="frFR",itIT="itIT",koKR="koKR",ptBR="ptBR",ruRU="ruRU",zhCN="zhCN",zhTW="zhTW"}
local allow_global_operations = false
local resolutions = { GetScreenResolutions() }
local graphicsAPIs = { GetGraphicsAPIs() }
local post_process_anti_aliasings = {"None","FXAA Low","FXAA High","CMAA"}
local resample_qualities = {L["Bilinear"],L["Bicubic"]}
local screenshot_formats = { jpg = "jpg", jpeg = "jpeg", tga = "tga"}
local MSAA_qualities = { ["0"] = L["None"], ["1,0"]=L["Color 2x / Depth 2x"],["2,0"]=L["Color 4x / Depth 4x"],
				["2,1"]=L["Color 4x / Depth 8x"],["2,2"]=L["Color 4x / Depth 16x"],
				["3,0"]=L["Color 8x / Depth 8x"],["3,1"]=L["Color 8x / Depth 16x"],["3,2"]=L["Color 8x / Depth 32x"]}
local sound_channels = {["24"]=L["Low (24)"],["32"]=L["Medium (32)"],["64"]=L["High (64)"]}
				
local scloak
local function showCloak(info,val)
	scloak = val
	ShowCloak(val)
end
local function showingCloak()
	if scloak == nil then
		scloak=ShowingCloak()
	end
	return scloak
end
local shelm
local ShowHelm = ShowHelm
local function showHelm(info,val)
	shelm = val
	ShowHelm(val)
end
local function showingHelm()
	if shelm == nil then
		shelm=ShowingHelm()
	end
	return shelm
end

local function get_output_drivers_table()
	local r = {}
	local t = Sound_GameSystem_GetNumOutputDrivers()
	for i=1,t do
		local s = Sound_GameSystem_GetOutputDriverNameByIndex(i-1)
		r[s] = s
	end
	return r
end
				
local function tablefind(array,key)
	local i = 1
	for i = 1,#array do
		if key == array[i] then
			return i
		end
	end
	return i
end

local function SetCVar_Global(key,value)
	if allow_global_operations == true then
		SetCVar(key,value)
	end
	allow_global_operations = false
end
local function SetCVar_Global_RestartGx(key,value)
	if allow_global_operations == true then
		SetCVar(key,value)
		RestartGx()
	end
	allow_global_operations = false
end
local options =
{
	type = "group",
	order = get_order(),
	name = "ControlPanel",
	args =
	{
		Global =
		{
			order = get_order(),
			name = L["Global"],
			desc = L["Requires a client restart."].."\n\n"..L["ControlPanel won't manage these options."],
			type = "group",
			args =
			{
				Allow_Once =
				{
					order = get_order(),
					name = L["Allow Once"],
					desc = L["Prevent misoperations on Global Settings"],
					type = "toggle",
					set = function(info,val)
						allow_global_operations = val
					end,
					get  = function(info)
						return allow_global_operations
					end,
					width = "full",
				},
				Display =
				{
					order = get_order(),
					name = L["Display"],
					desc = L["Requires a restart of the game's graphic subsystem"],
					type = "group",
					args =
					{
						gxWindow =
						{
							order = get_order(),
							name = L["Windowed"],
							type = "toggle",
							set = function(info,val)
								SetCVar_Global_RestartGx("gxWindow",val)
							end,
							get = function(info)
								return GetCVarBool("gxWindow")
							end,
						},
						gxMaximize =
						{
							order = get_order(),
							name = L["Fullscreen"],
							type = "toggle",
							set = function(info,val)
								SetCVar_Global_RestartGx("gxMaximize",val)
							end,
							get = function(info)
								return GetCVarBool("gxMaximize")
							end,
						},
						gxResolution =
						{
							order = get_order(),
							name = L["Resolution"],
							desc = L["Higher resolution will result in increased clarity, but this greatly affects performance. Choose a resolution that matches the aspect ratio of your monitor."],
							type = "select",
							values = resolutions,
							set = function(info,val)
								SetCVar_Global_RestartGx("gxResolution",resolutions[val])
							end,
							get = function(info)
								return GetCurrentResolution()
							end,
						},
						gxApi =
						{
							order = get_order(),
							name = L["Graphics API"],
							desc = L["Requires a client restart."],
							type = "select",
							values = graphicsAPIs,
							set = function(info,val)
								SetCVar_Global("gxApi",graphicsAPIs[val])
							end,
							get = function(info)
								return tablefind(graphicsAPIs,GetCVar("gxApi"))
							end,
						},
						violenceLevel =
						{
							order = get_order(),
							name = L["Violence Level"],
							type = "range",
							min = 0,
							max = 5,
							step = 1,
							set = function(info,val)
								SetCVar_Global("violenceLevel",val)
							end,
							get = function(info)
								return get_cvar_number("violenceLevel")
							end,
						},
					}
				},
				Langauges =
				{
					order = get_order(),
					name = L["Langauges"],
					type = "group",
					args =
					{
						portal = 
						{
							order = get_order(),
							name = string_format(L["Portal : %s"],string.upper(GetCVar("portal"))),
							type = "description",
							fontSize = "large",
							width = "full",
						},
						textLocale =
						{
							order = get_order(),
							name = L["Text"],
							type = "select",
							values = languages,
							set = function(info,val)
								if val ~= "zhCN" then
									SetCVar_Global("textLocale",val)
								end
							end,
							get = function(info)
								return GetCVar("textLocale")
							end,
						},
						audioLocale =
						{
							order = get_order(),
							name = L["Audio"],
							type = "select",
							values = languages,
							set = function(info,val)
								if val ~= "zhCN" then
									SetCVar_Global("audioLocale",val)
								end
							end,
							get = function(info)
								return GetCVar("audioLocale")
							end,
						},
						installLocale =
						{
							order = get_order(),
							name = L["Install"],
							type = "select",
							values = languages,
							set = function(info,val)
								if val ~= "zhCN" then
									SetCVar_Global("installLocale",val)
								end
							end,
							get = function(info)
								return GetCVar("installLocale")
							end,
						},
					},
				},
			}
		},
		Interface =
		{
			order = get_order(),
			name = L["Interface"],
			desc = L["ControlPanel won't manage these options."],
			type = "group",
			args =
			{
				Armory =
				{
					name = L["Armory"],
					type = "group",
					args =
					{
						cloak =
						{
							order = get_order(),
							name = L["Cloak"],
							type = "toggle",
							set = showCloak,
							get = showingCloak
						},
						helm =
						{
							order = get_order(),
							name = L["Helm"],
							type = "toggle",
							set = showHelm,
							get = showingHelm
						}
					}
				},
				Camera =
				{
					name = L["Camera"],
					desc = L["These options affect whether certain UI and character elements are hidden or displayed."],
					type = "group",
					args =
					{
						Distance =
						{
							order = get_order(),
							name = L["Distance"],
							type = "group",
							args =
							{
								cameraDistanceMax =
								{
									order = get_order(),
									name = L["Max"],
									type = "range",
									min = 0,
									max = 50,
									set = function(info,val)
										SetCVar("cameraDistanceMax",val)
									end,
									get = function(info)
										return get_cvar_number("cameraDistanceMax")
									end,
								},
								cameraDistanceMaxFactor =
								{
									order = get_order(),
									name = L["Max Factor"],
									type = "range",
									min = 0,
									max = 2,
									set = function(info,val)
										SetCVar("cameraDistanceMaxFactor",val)
									end,
									get = function(info)
										return get_cvar_number("cameraDistanceMaxFactor")
									end,
								},
								cameraDistanceMoveSpeed =
								{
									order = get_order(),
									name = L["Move Speed"],
									type = "range",
									min =  0,
									max = 50,
									set = function(info,val)
										SetCVar("cameraDistanceMoveSpeed",val)
									end,
									get = function(info)
										return get_cvar_number("cameraDistanceMoveSpeed")
									end,
								}
							}
						}
					}
				},


			}
		},
		Graphics =
		{
			order = get_order(),
			name = L["Graphics"],
			type = "group",
			args =
			{
				hdPlayerModels =
				{
					order = get_order(),
					name = L["Show New Character Models"],
					desc = L["Check this to show new higher definition character models. (64-bit OS recommended)"],
					type = "toggle",
					set = function(info,val)
						ControlPanel:SetCVar("hdPlayerModels",val)
					end,
					get = function(info)
						return ControlPanel:GetCVarBool("hdPlayerModels")
					end,
					width = "full",
				},
				useUiScale =
				{
					order = get_order(),
					name = L["Use UI Scale"],
					desc = L["Check to use UI Scale Slider, uncheck to use the system default scale."],
					type = "toggle",
					set = function(info,val)
						ControlPanel:SetCVar("useUiScale",val)
					end,
					get = function(info)
						return ControlPanel:GetCVarBool("useUiScale")
					end,
				},
				uiScale =
				{
					order = get_order(),
					name = L["UI Scale"],
					desc = L["Changes the size of the game's user interface"],
					type = "range",
					min = 0,
					max = 10,
					set = function(info,val)
						ControlPanel:SetCVar("uiScale",val)
					end,
					get = function(info)
						return ControlPanel:GetCVar("uiScale")
					end,
				},
				Display =
				{
					order = get_order(),
					name = L["Display"],
					type = "group",
					args =
					{
						gxVSync =
						{
							order = get_order(),
							name = L["Vertical Sync"],
							desc = L["Synchronizes your framerate to some fraction of your monitor's refresh rate. Enable this if you see excessive screen tearing in game."],
							type = "toggle",
							set = function(info,val)
								ControlPanel:SetCVar("gxVSync",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarBool("gxVSync")
							end,
						},
						gxTripleBuffer =
						{
							order = get_order(),
							name = L["Triple Buffering"],
							desc = L["Enables triple buffering of frames when vertical sync is enabled. Selecting triple buffering may even out frame rate spikes, but may also cause slight input lag."],
							type = "toggle",
							set = function(info,val)
								ControlPanel:SetCVar("gxTripleBuffer",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarBool("gxTripleBuffer")
							end,
						},
						gxFixLag =
						{
							order = get_order(),
							name = L["Reduce Input Lag"],
							desc = L["Enabling this reduces user interface lag, but may drastically reduce frame rates."],
							type = "toggle",
							set = function(info,val)
								ControlPanel:SetCVar("gxFixLag",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarBool("gxFixLag")
							end,
						},
						gxCursor = 
						{
							order = get_order(),
							name = L["Hardware Cursor"],
							desc = L["Enable this option for a more responsive cursor unless you have cursor problems."],
							type = "toggle",
							set = function(info,val)
								ControlPanel:SetCVar("gxCursor",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarBool("gxCursor")
							end,
						},
						maxFPS = 
						{
							order = get_order(),
							name = L["Max Foreground FPS"],
							desc = L["Maximum framerate while running in the foreground\n\n(0 represents no limits)"],
							type = "range",
							min = 0,
							max = 200,
							set = function(info,val)
								ControlPanel:SetCVar("maxFPS",val)
							end,
							get = function(info)
								return ControlPanel:GetCVar("maxFPS")
							end,
						},
						maxFPSBk = 
						{
							order = get_order(),
							name = L["Max Background FPS"],
							desc = L["Maximum framerate while running in the background\n\n(0 represents no limits)"],
							type = "range",
							min = 0,
							max = 200,
							set = function(info,val)
								ControlPanel:SetCVar("maxFPSBk",val)
							end,
							get = function(info)
								return ControlPanel:GetCVar("maxFPSBk")
							end,
						},
					}
				},
				Anti_Aliasing =
				{
					order = get_order(),
					name = L["Anti-Aliasing"],
					desc = L["Anti-Aliasing is a technique to smooth out jaggy edges, especially at low resolutions, but can result in a slightly blurred image. Enabling Anti-Aliasing will reduce performance."],
					type = "group",
					args =
					{
						ffxAntiAliasingMode =
						{
							order = get_order(),
							name = L["Post-Process AA"],
							type = "select",
							values = post_process_anti_aliasings,
							set = function(info,val)
								ControlPanel:SetCVar("ffxAntiAliasingMode",val-1)
							end,
							get = function(info)
								return ControlPanel:GetCVar("ffxAntiAliasingMode")+1
							end,
							width = "full",
						},
						MSAA =
						{
							order = get_order(),
							name = "MSAA",
							desc = L["Multisample anti-aliasing"],
							type = "group",
							args =
							{
								MSAAQuality =
								{
									order = get_order(),
									name = "MSAA",
									desc = L["Sets the Multisample Anti-Aliasing mode. Both the number of color and depth samples can be specified."],
									type = "select",
									values = MSAA_qualities,
									set = function(info,val)
										ControlPanel:SetCVar("MSAAQuality",val)
									end,
									get = function(info)
										return ControlPanel:GetCVarString("MSAAQuality")
									end,
								},
								MSAAAlphaTest =
								{
									order = get_order(),
									name = L["Alpha-Test"],
									desc = L["Evaluates alpha-test operation at sample frequency rather than pixel frequency when MSAA is enabled."],
									type = "toggle",
									set = function(info,val)
										ControlPanel:SetCVar("MSAAAlphaTest",val)
									end,
									get = function(info)
										return ControlPanel:GetCVarBool("MSAAAlphaTest")
									end,
								}
							}
						},
						SSAA =
						{
							order = get_order(),
							name = "SSAA",
							desc = L["Supersampling Anti-aliasing"],
							type = "group",
							args =
							{
								RenderScale =
								{
									order = get_order(),
									name = L["Render Scale"],
									desc = L["Multiplier on display resolution for rendering the 3D World."],
									type = "range",
									min = 0,
									max = 2,
									set = function(info,val)
										ControlPanel:SetCVar("RenderScale",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("RenderScale")
									end,
								},
								ResampleQuality =
								{
									order = get_order(),
									name = L["Resample Quality"],
									desc = L["When using a Render Scale that is not 100%, this adjusts the quality of the filter to display resolution."],
									type = "select",
									values = resample_qualities,
									set = function(info,val)
										ControlPanel:SetCVar("ResampleQuality",val-1)
									end,
									get = function(info)
										return ControlPanel:GetCVar("ResampleQuality")+1
									end,
								}
							}
						},
					}
				},
				BasicSettings = 
				{
					order = get_order(),
					name = L["Basic Settings"],
					type = "group",
					args = 
					{
						graphicsQuality = 
						{
							order = get_order(),
							name = L["Graphics Quality"],
							type = "range",
							min = 0,
							max = 6,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("graphicsQuality",val)
								set_graphics_quality(val)
							end,
							get = function(info)
								return ControlPanel:GetCVar("graphicsQuality")
							end,
							width = "full",
						},
						Textures =
						{
							order = get_order(),
							name = L["Textures"],
							type = "group",
							args =
							{
								terrainmiplevel = 
								{
									order = get_order(),
									name = L["Terrain Blending"],
									desc = L["Controls the level of terrain blending. (Lower is better.) Increasing this may slightly improve performance."],
									type = "range",
									min = 0,
									max = 1,
									set = function(info,val)
										ControlPanel:SetCVar("terrainmiplevel",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("terrainmiplevel")
									end,
								},
								worldbasemip = 
								{
									order = get_order(),
									name = L["Environment and Character Resolution"],
									desc = L["Controls the level of environment and character resolution. (Lower is better.) Increasing this may slightly improve performance."],
									type = "range",
									min = 0,
									max = 2,
									set = function(info,val)
										ControlPanel:SetCVar("worldbasemip",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("worldbasemip")
									end,
								},
								textureFilteringMode = 
								{
									order = get_order(),
									name = L["Texture Filtering"],
									desc = L["Increases texture sharpness, particularly for textures view at an angle. Decrease to improve performance.\n\nBilinear(0)\nTrilinear(1)\n2x Anisotropic(2)\n4x Anisotropic(3)\n8x Anisotropic(4)\n16x Anisotropic(5)\n"],
									type = "range",
									min = 0,
									max = 5,
									step = 1,
									set = function(info,val)
										ControlPanel:SetCVar("textureFilteringMode",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("textureFilteringMode")
									end,
								},
								ProjectedTextures = 
								{
									order = get_order(),
									name = L["Projected Textures"],
									desc = L["Enables the projecting of textures to the environment. Disabling this may greatly improve performance."],
									type = "toggle",
									set = function(info,val)
										ControlPanel:SetCVar("ProjectedTextures",val)
									end,
									get = function(info)
										return ControlPanel:GetCVarBool("ProjectedTextures")
									end,
								}
							}
						},
						Environment =
						{
							order = get_order(),
							name = L["Environment"],
							type = "group",
							args = 
							{
								farclip = 
								{
									order = get_order(),
									name = L["View Distance"],
									desc = L["View distance controls how far you can see. Larger view distances require more memory and a faster processor."],
									type = "range",
									min = 0,
									max = 1300,
									step = 0.0001,
									set = function(info,val)
										ControlPanel:SetCVar("farclip",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("farclip")
									end,
								},
								environmentDetail = 
								{
									order = get_order(),
									name = L["Environment Detail"],
									desc = L["Controls how far you can see objects. Decrease to improve performance."],
									type = "range",
									min = 50,
									max = 150,
									set = function(info,val)
										ControlPanel:SetCVar("environmentDetail",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("environmentDetail")
									end,
								},
								groundEffectDensity = 
								{
									order = get_order(),
									name = L["Ground Clutter Density"],
									desc = L["Controls the density at which ground clutter items, like grass and foliage, are placed. Decrease to improve performance."],
									type = "range",
									min = 16,
									max = 128,
									set = function(info,val)
										ControlPanel:SetCVar("groundEffectDensity",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("groundEffectDensity")
									end,
								},
								groundEffectDist = 
								{
									order = get_order(),
									name = L["Ground Clutter Distance"],
									desc = L["Controls the distance at which ground clutter items, like grass and foliage, are placed. Decrease to improve performance."],
									type = "range",
									min = 50,
									max = 260,
									set = function(info,val)
										ControlPanel:SetCVar("groundEffectDist",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("groundEffectDist")
									end,
								},
							}
						},
						Effects =
						{
							order = get_order(),
							name = L["Effects"],
							type = "group",
							args = 
							{
								shadowMode = 
								{
									order = get_order(),
									name = L["Shadow Quality"],
									desc = L["Controls both the method and quality of shadows. Decreasing this may greatly improve performance."],
									type = "range",
									min = 0,
									max = 4,
									step = 1,
									set = function(info,val)
										ControlPanel:SetCVar("shadowMode",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("shadowMode")
									end,
								},
								waterDetail = 
								{
									order = get_order(),
									name = L["Liquid Detail"],
									desc = L["Controls the rendering quality of liquid. Decreasing this may improve performance.\n\nLow(0): Animated Liquid textures, texture based ripples and no reflection.\n\nFair(1): Normalmap liquid textures, texture based ripples and sky reflection.\n\nGood(2): Normalmap liquid textures, procedural ripples and screen-based reflection.\n\nUltra(3): Normalmap liquid textures, procedural ripples and full reflection."],
									type = "range",
									min = 0,
									max = 3,
									step = 1,
									set = function(info,val)
										ControlPanel:SetCVar("waterDetail",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("waterDetail")
									end,
								},
								sunShafts = 
								{
									order = get_order(),
									name = L["Sunshafts"],
									desc = L["Controls the method and quality used to display sunshafts and glare effects. Disabling this setting may improve performance.\n\nDisabled(0): Traditional sunshafts and glave.\n\nLow(1): Depth-based sunshafts and glare.\n\nHigh(2): Depth-based sunshafts and glare with improved sampling to reduce anti-aliasing."],
									type = "range",
									min = 0,
									max = 2,
									step = 1,
									set = function(info,val)
										ControlPanel:SetCVar("sunShafts",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("sunShafts")
									end,
								},
								particleDensity = 
								{
									order = get_order(),
									name = L["Particle Density"],
									desc = L["Controls the number of particles used in effects caused by spells, fires, etc. Decrease to improve performance."],
									type = "range",
									min = 10,
									max = 100,
									set = function(info,val)
										ControlPanel:SetCVar("particleDensity",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("particleDensity")
									end,
								},
								SSAO = 
								{
									order = get_order(),
									name = L["SSAO"],
									desc = L["Controls the rendering quality of the advanced lighting effects. Decreasing this may greatly improve performance."],
									type = "range",
									min = 0,
									max = 3,
									step = 1,
									set = function(info,val)
										ControlPanel:SetCVar("SSAO",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("SSAO")
									end,
								},
								lightmode = 
								{
									order = get_order(),
									name = L["Lightning Quality"],
									desc = L["Controls both the method and quality of lighting. Decreasing this may improve performance."],
									type = "range",
									min = 0,
									max = 2,
									step = 1,
									set = function(info,val)
										ControlPanel:SetCVar("lightmode",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("lightmode")
									end,
								},
								DepthBasedOpacity = 
								{
									order = get_order(),
									name = L["Particle Depth Fading"],
									type = "toggle",
									set = function(info,val)
										ControlPanel:SetCVar("DepthBasedOpacity",val)
									end,
									get = function(info)
										return ControlPanel:GetCVarBool("DepthBasedOpacity")
									end,
								},
								refraction = 
								{
									order = get_order(),
									name = L["Refraction"],
									type = "range",
									min = 0,
									max = 2,
									step = 1,
									set = function(info,val)
										ControlPanel:SetCVar("refraction",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("refraction")
									end,
								},
								outlineEngineMode = 
								{
									order = get_order(),
									name = L["Outline Mode"],
									desc = L["Controls whether the selection outline effect is allowed. See the interface Display panel for more options."],
									type = "toggle",
									set = function(info,val)
										ControlPanel:SetCVar("outlineEngineMode",val)
									end,
									get = function(info)
										return ControlPanel:GetCVarBool("outlineEngineMode")
									end,
								}
							}
						}
					}
				},
				
				RaidAndBattlegroundSettings = 
				{
					order = get_order(),
					name = L["Raid And Battleground Settings"],
					type = "group",
					args = 
					{
						graphicsQuality = 
						{
							order = get_order(),
							name = L["Graphics Quality"],
							type = "range",
							min = 0,
							max = 6,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("RAIDgraphicsQuality",val)
								set_raid_graphics_quality(val)
							end,
							get = function(info)
								return ControlPanel:GetCVar("RAIDgraphicsQuality")
							end,
							width = "full",
						},
						RAIDsettingsEnabled =
						{
							order = get_order(),
							name = L["Enable"],
							desc = L["Check this option to enable alternate graphic setting for Raids and Battlegrounds"],
							type = "toggle",
							set = function(info,val)
								ControlPanel:SetCVar("RAIDsettingsEnabled",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarBool("RAIDsettingsEnabled")
							end,
							width = "full",
						},
						Textures =
						{
							order = get_order(),
							name = L["Textures"],
							type = "group",
							args =
							{
								terrainmiplevel = 
								{
									order = get_order(),
									name = L["Terrain Blending"],
									desc = L["Controls the level of terrain blending. (Lower is better.) Increasing this may slightly improve performance."],
									type = "range",
									min = 0,
									max = 1,
									set = function(info,val)
										ControlPanel:SetCVar("RAIDerrainmiplevel",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("RAIDterrainmiplevel")
									end,
								},
								worldbasemip = 
								{
									order = get_order(),
									name = L["Environment and Character Resolution"],
									desc = L["Controls the level of environment and character resolution. (Lower is better.) Increasing this may slightly improve performance."],
									type = "range",
									min = 0,
									max = 2,
									set = function(info,val)
										ControlPanel:SetCVar("RAIDworldbasemip",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("RAIDworldbasemip")
									end,
								},
								textureFilteringMode = 
								{
									order = get_order(),
									name = L["Texture Filtering"],
									desc = L["Increases texture sharpness, particularly for textures view at an angle. Decrease to improve performance.\n\nBilinear(0)\nTrilinear(1)\n2x Anisotropic(2)\n4x Anisotropic(3)\n8x Anisotropic(4)\n16x Anisotropic(5)\n"],
									type = "range",
									min = 0,
									max = 5,
									step = 1,
									set = function(info,val)
										ControlPanel:SetCVar("RAIDtextureFilteringMode",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("RAIDtextureFilteringMode")
									end,
								},
								ProjectedTextures = 
								{
									order = get_order(),
									name = L["Projected Textures"],
									desc = L["Enables the projecting of textures to the environment. Disabling this may greatly improve performance."],
									type = "toggle",
									set = function(info,val)
										ControlPanel:SetCVar("RAIDProjectedTextures",val)
									end,
									get = function(info)
										return ControlPanel:GetCVarBool("RAIDProjectedTextures")
									end,
								}
							}
						},
						Environment =
						{
							order = get_order(),
							name = L["Environment"],
							type = "group",
							args = 
							{
								farclip = 
								{
									order = get_order(),
									name = L["View Distance"],
									desc = L["View distance controls how far you can see. Larger view distances require more memory and a faster processor."],
									type = "range",
									min = 0,
									max = 1300,
									step = 0.0001,
									set = function(info,val)
										ControlPanel:SetCVar("RAIDfarclip",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("RAIDfarclip")
									end,
								},
								environmentDetail = 
								{
									order = get_order(),
									name = L["Environment Detail"],
									desc = L["Controls how far you can see objects. Decrease to improve performance."],
									type = "range",
									min = 50,
									max = 150,
									set = function(info,val)
										ControlPanel:SetCVar("RAIDenvironmentDetail",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("RAIDenvironmentDetail")
									end,
								},
								groundEffectDensity = 
								{
									order = get_order(),
									name = L["Ground Clutter Density"],
									desc = L["Controls the density at which ground clutter items, like grass and foliage, are placed. Decrease to improve performance."],
									type = "range",
									min = 16,
									max = 128,
									set = function(info,val)
										ControlPanel:SetCVar("RAIDgroundEffectDensity",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("RAIDgroundEffectDensity")
									end,
								},
								groundEffectDist = 
								{
									order = get_order(),
									name = L["Ground Clutter Distance"],
									desc = L["Controls the distance at which ground clutter items, like grass and foliage, are placed. Decrease to improve performance."],
									type = "range",
									min = 50,
									max = 260,
									set = function(info,val)
										ControlPanel:SetCVar("RAIDgroundEffectDist",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("RAIDgroundEffectDist")
									end,
								},
							}
						},
						Effects =
						{
							order = get_order(),
							name = L["Effects"],
							type = "group",
							args = 
							{
								shadowMode = 
								{
									order = get_order(),
									name = L["Shadow Quality"],
									desc = L["Controls both the method and quality of shadows. Decreasing this may greatly improve performance."],
									type = "range",
									min = 0,
									max = 4,
									step = 1,
									set = function(info,val)
										ControlPanel:SetCVar("RAIDshadowMode",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("RAIDshadowMode")
									end,
								},
								waterDetail = 
								{
									order = get_order(),
									name = L["Liquid Detail"],
									desc = L["Controls the rendering quality of liquid. Decreasing this may improve performance.\n\nLow(0): Animated Liquid textures, texture based ripples and no reflection.\n\nFair(1): Normalmap liquid textures, texture based ripples and sky reflection.\n\nGood(2): Normalmap liquid textures, procedural ripples and screen-based reflection.\n\nUltra(3): Normalmap liquid textures, procedural ripples and full reflection."],
									type = "range",
									min = 0,
									max = 3,
									step = 1,
									set = function(info,val)
										ControlPanel:SetCVar("RAIDwaterDetail",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("RAIDwaterDetail")
									end,
								},
								sunShafts = 
								{
									order = get_order(),
									name = L["Sunshafts"],
									desc = L["Controls the method and quality used to display sunshafts and glare effects. Disabling this setting may improve performance.\n\nDisabled(0): Traditional sunshafts and glave.\n\nLow(1): Depth-based sunshafts and glare.\n\nHigh(2): Depth-based sunshafts and glare with improved sampling to reduce anti-aliasing."],
									type = "range",
									min = 0,
									max = 2,
									step = 1,
									set = function(info,val)
										ControlPanel:SetCVar("RAIDsunShafts",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("RAIDsunShafts")
									end,
								},
								particleDensity = 
								{
									order = get_order(),
									name = L["Particle Density"],
									desc = L["Controls the number of particles used in effects caused by spells, fires, etc. Decrease to improve performance."],
									type = "range",
									min = 10,
									max = 100,
									set = function(info,val)
										ControlPanel:SetCVar("RAIDparticleDensity",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("RAIDparticleDensity")
									end,
								},
								SSAO = 
								{
									order = get_order(),
									name = L["SSAO"],
									desc = L["Controls the rendering quality of the advanced lighting effects. Decreasing this may greatly improve performance."],
									type = "range",
									min = 0,
									max = 3,
									step = 1,
									set = function(info,val)
										ControlPanel:SetCVar("RAIDSSAO",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("RAIDSSAO")
									end,
								},
								lightmode = 
								{
									order = get_order(),
									name = L["Lightning Quality"],
									desc = L["Controls both the method and quality of lighting. Decreasing this may improve performance."],
									type = "range",
									min = 0,
									max = 2,
									step = 1,
									set = function(info,val)
										ControlPanel:SetCVar("RAIDlightmode",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("RAIDlightmode")
									end,
								},
								DepthBasedOpacity = 
								{
									order = get_order(),
									name = L["Particle Depth Fading"],
									type = "toggle",
									set = function(info,val)
										ControlPanel:SetCVar("RAIDDepthBasedOpacity",val)
									end,
									get = function(info)
										return ControlPanel:GetCVarBool("RAIDDepthBasedOpacity")
									end,
								},
								refraction = 
								{
									order = get_order(),
									name = L["Refraction"],
									type = "range",
									min = 0,
									max = 2,
									step = 1,
									set = function(info,val)
										ControlPanel:SetCVar("RAIDrefraction",val)
									end,
									get = function(info)
										return ControlPanel:GetCVar("RAIDrefraction")
									end,
								},
								outlineEngineMode = 
								{
									order = get_order(),
									name = L["Outline Mode"],
									desc = L["Controls whether the selection outline effect is allowed. See the interface Display panel for more options."],
									type = "toggle",
									set = function(info,val)
										ControlPanel:SetCVar("RAIDoutlineEngineMode",val)
									end,
									get = function(info)
										return ControlPanel:GetCVarBool("RAIDoutlineEngineMode")
									end,
								}
							}
						}
					}
				},

				Screenshot =
				{
					order = get_order(),
					name = L["Screenshot"],
					type = "group",
					args = 
					{
						screenshotFormat =
						{
							order = get_order(),
							name = L["Format"],
							type = "select",
							values = screenshot_formats,
							set = function(info,val)
								ControlPanel:SetCVar("screenshotFormat",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarString("screenshotFormat")
							end
						},
						screenshotQuality =
						{
							order = get_order(),
							name = L["Quality"],
							desc = L["Note this option only affects JPEG screenshots (TGA screenshots always use lossless quality)."],
							type = "range",
							min = 1,
							max =10,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("screenshotQuality",val)
							end,
							get = function(info)
								return ControlPanel:GetCVar("screenshotQuality")
							end,
						},
					}
				}
			}
		},
		Network =
		{
			order = get_order(),
			name = L["Network"],
			type = "group",
			args =
			{
				disableServerNagle = 
				{
					order = get_order(),
					name = L["Optimize Network for Speed"],
					desc = L["Enable this option to optimize network usage for speed in order to decrease lag.\nThis will increase your bandwidth usage, and may cause problems in some environments. Disabling this option may help resolve certain disconnection issues."],
					type = "toggle",
					set = function(info,val)
						ControlPanel:SetCVar("disableServerNagle",val)
					end,
					get = function(info)
						return ControlPanel:GetCVarBool("disableServerNagle")
					end,
					width = "full",
				},
				useIPv6 =
				{
					order = get_order(),
					name = L["Enable IPv6 when available"],
					desc = L["Enable the use of IPv6, the technology behind the next-generation Internet.\nRequires IPv6 connectivity to the Internet. Checking this box without IPv6 connectivity may prevent you from playing WoW."],
					type = "toggle",
					set = function(info,val)
						ControlPanel:SetCVar("useIPv6",val)
					end,
					get = function(info)
						return ControlPanel:GetCVarBool("useIPv6")
					end,
					width = "full",
				},
				advancedCombatLogging =
				{
					order = get_order(),
					name = L["Advanced Combat Logging"],
					desc = L["Enable extra logging data when writing the combat log to a text file.\nThis may cause additional network lag when using the \"combatlog\" chat command."]..L["\n\nControlPanel won't manage this option."],
					type = "toggle",
					set = function(info,val)
						SetCVar("advancedCombatLogging",val)
					end,
					get = function(info)
						return get_cvar_bool("advancedCombatLogging")
					end,
					width = "full",
				},
			}
		},
		Sound =
		{
			order = get_order(),
			name = L["Sound"],
			desc = L["ControlPanel only manages Volumes and Class Voices."],
			type = "group",
			args =
			{
				Sound_EnableAllSound =
				{
					order = get_order(),
					name = L["Enable"],
					desc = L["Enables or disables all sound."],
					type = "toggle",
					set = function(info,val)
						SetCVar("Sound_EnableAllSound",val)
					end,
					get = function(info)
						return get_cvar_bool("Sound_EnableAllSound")
					end,
					width = "full",
				},
				Sound_MasterVolume =
				{
					order = get_order(),
					name = L["Master Volume"],
					type = "range",
					min = 0,
					max = 1,
					isPercent = true,
					set = function(info,val)
						ControlPanel:SetCVar("Sound_MasterVolume",val)
					end,
					get = function(info)
						return ControlPanel:GetCVar("Sound_MasterVolume")
					end,
					width = "full",
				},
				SoundEffects =
				{
					order = get_order(),
					name = L["Sound Effects"],
					type = "group",
					args = 
					{
						Sound_EnableSFX = 
						{
							order = get_order(),
							name = L["Enable"],
							type = "toggle",
							set = function(info,val)
								SetCVar("Sound_EnableSFX",val)
							end,
							get = function(info)
								return get_cvar_bool("Sound_EnableSFX")
							end,
							width = "full",
						},
						Sound_SFXVolume =
						{
							order = get_order(),
							name = L["Volume"],
							type = "range",
							min = 0,
							max = 1,
							isPercent = true,
							set = function(info,val)
								ControlPanel:SetCVar("Sound_SFXVolume",val)
							end,
							get = function(info)
								return ControlPanel:GetCVar("Sound_SFXVolume")
							end,
							width = "full",
						},
						Sound_EnablePetSounds =
						{
							order = get_order(),
							name = L["Pet"],
							desc = L["Toggles pet idle and aggro sounds on and off."],
							type = "toggle",
							set = function(info,val)
								SetCVar("Sound_EnablePetSounds",val)
							end,
							get = function(info)
								return get_cvar_bool("Sound_EnablePetSounds")
							end,
						},
						Sound_EnableEmoteSounds =
						{
							order = get_order(),
							name = L["Emote"],
							type = "toggle",
							set = function(info,val)
								SetCVar("Sound_EnableEmoteSounds",val)
							end,
							get = function(info)
								return get_cvar_bool("Sound_EnableEmoteSounds")
							end,
						},
					}
				},
				Music =
				{
					order = get_order(),
					name = L["Music"],
					type = "group",
					args = 
					{
						Sound_EnableMusic =
						{
							order = get_order(),
							name = L["Enable"],
							type = "toggle",
							set = function(info,val)
								SetCVar("Sound_EnableMusic",val)
							end,
							get = function(info)
								return get_cvar_bool("Sound_EnableMusic")
							end,
							width = "full",
						},
						Sound_MusicVolume =
						{
							order = get_order(),
							name = L["Volume"],
							type = "range",
							min = 0,
							max = 1,
							isPercent = true,
							set = function(info,val)
								ControlPanel:SetCVar("Sound_MusicVolume",val)
							end,
							get = function(info)
								return ControlPanel:GetCVar("Sound_MusicVolume")
							end,
							width = "full",
						},
						Sound_ZoneMusicNoDelay = 
						{
							order = get_order(),
							name = L["Loop"],
							type = "toggle",
							set = function(info,val)
								SetCVar("Sound_ZoneMusicNoDelay",val)
							end,
							get = function(info)
								return get_cvar_bool("Sound_ZoneMusicNoDelay")
							end,
						},
						Sound_EnablePetBattleMusic = 
						{
							order = get_order(),
							name = L["Pet Battle"],
							type = "toggle",
							set = function(info,val)
								SetCVar("Sound_EnablePetBattleMusic",val)
							end,
							get = function(info)
								return get_cvar_bool("Sound_EnablePetBattleMusic")
							end,
						},
					}
				},
				Ambience = 
				{
					order = get_order(),
					name = L["Ambience"],
					type = "group",
					args = 
					{
						Sound_EnableAmbience =
						{
							order = get_order(),
							name = L["Enable"] ,
							type = "toggle",
							set = function(info,val)
								SetCVar("Sound_EnableAmbience",val)
							end,
							get = function(info)
								return get_cvar_bool("Sound_EnableAmbience")
							end,
							width = "full",
						},
						Sound_AmbienceVolume =
						{
							order = get_order(),
							name = L["Volume"],
							type = "range",
							min = 0,
							max = 1,
							isPercent = true,
							set = function(info,val)
								ControlPanel:SetCVar("Sound_AmbienceVolume",val)
							end,
							get = function(info)
								return ControlPanel:GetCVar("Sound_AmbienceVolume")
							end,
							width = "full",
						},
					}
				},
				Dialog =
				{
					order = get_order(),
					name = L["Dialog"],
					type = "group",
					args = 
					{
						Sound_EnableDialog =
						{
							order = get_order(),
							name = L["Enable"] ,
							type = "toggle",
							set = function(info,val)
								SetCVar("Sound_EnableDialog",val)
							end,
							get = function(info)
								return get_cvar_bool("Sound_EnableDialog")
							end,
							width = "full",
						},
						Sound_DialogVolume =
						{
							order = get_order(),
							name = L["Volume"],
							type = "range",
							min = 0,
							max = 1,
							isPercent = true,
							set = function(info,val)
								ControlPanel:SetCVar("Sound_DialogVolume",val)
							end,
							get = function(info)
								return ControlPanel:GetCVar("Sound_DialogVolume")
							end,
							width = "full",
						},
						Sound_EnableErrorSpeech = 
						{
							order = get_order(),
							name = L["Error Speech"] ,
							type = "toggle",
							set = function(info,val)
								SetCVar("Sound_EnableErrorSpeech",val)
							end,
							get = function(info)
								return get_cvar_bool("Sound_EnableErrorSpeech")
							end,
						},
						movieSubtitle = 
						{
							order = get_order(),
							name = L["Cinematic Subtitles"],
							type = "toggle",
							set = function(info,val)
								SetCVar("movieSubtitle",val)
							end,
							get = function(info)
								return get_cvar_bool("movieSubtitle")
							end,
						},
					}
				},
				Playback =
				{
					order = get_order(),
					name = L["Playback"],
					desc = L["ControlPanel won't manage these options."],
					type = "group",
					args = 
					{
						Sound_EnableSoundWhenGameIsInBG = 
						{
							order = get_order(),
							name = L["Sound in Background"],
							desc = L["Enable to allow sounds to play even when World of Warcraft is in the background."],
							type = "toggle",
							set = function(info,val)
								SetCVar("Sound_EnableSoundWhenGameIsInBG",val)
							end,
							get = function(info)
								return get_cvar_bool("Sound_EnableSoundWhenGameIsInBG")
							end,
							width = "full",
						},
						Sound_EnableReverb = 
						{
							order = get_order(),
							name = L["Reverb"],
							desc = L["This will affect performance."],
							type = "toggle",
							set = function(info,val)
								SetCVar("Sound_EnableReverb",val)
							end,
							get = function(info)
								return get_cvar_bool("Sound_EnableReverb")
							end,
							width = "full",
						},
						Sound_EnableSoftwareHRTF = 
						{
							order = get_order(),
							name = L["Headphone Mode"],
							desc = L["Enables software emulation for headphone surround sound."],
							type = "toggle",
							set = function(info,val)
								SetCVar("Sound_EnableSoftwareHRTF",val)
							end,
							get = function(info)
								return get_cvar_bool("Sound_EnableSoftwareHRTF")
							end,
							width = "full",
						},
					}
				},

				Hardware =
				{
					order = get_order(),
					name = L["Hardware"],
					desc = L["Requires a restart of the game's sound systems"] ,
					type = "group",
					args =
					{
						Sound_OutputDriverName =
						{
							order = get_order(),
							name = L["Game Sound Output"],
							type = "select",
							values = get_output_drivers_table,
							set = function(info,val)
								SetCVar("Sound_OutputDriverName",val)
								Sound_GameSystem_RestartSoundSystem()
							end,
							get = function(info)
								return GetCVar("Sound_OutputDriverName")
							end
						},
						Sound_NumChannels = 
						{
							order = get_order(),
							name = L["Sound Channels"],
							type = "select",
							values = sound_channels,
							set = function(info,val)
								SetCVar("Sound_NumChannels",val)
								Sound_GameSystem_RestartSoundSystem()
							end,
							get = function(info)
								return GetCVar("Sound_NumChannels")
							end
						}
					}
				},
				
				ClassVoices = 
				{
					order = get_order(),
					name = L["Class Voices"],
					desc = L["Special voice modulation effects for some classes. Disable to improve performance."],
					type = "group",
					args =
					{
						Sound_EnableDSPEffects = 
						{
							order = get_order(),
							name = GetClassInfoByID(6),
							type = "toggle",
							set = function(info,val)
								ControlPanel:SetCVar("Sound_EnableDSPEffects",val)
							end,	
							get = function(info)
								return ControlPanel:GetCVarBool("Sound_EnableDSPEffects")
							end,
						}
					}
				}
			}
		},
		Script =
		{
			order = get_order(),
			name = L["Script"],
			type = "group",
			args =
			{
				scriptErrors =
				{
					order = get_order(),
					name = L["Display Lua Error"],
					type = "toggle",
					set = function(info,val)
						ControlPanel:SetCVar("scriptErrors",val)
					end,
					get = function(info)
						return ControlPanel:GetCVarBool("scriptErrors")
					end,
				},
				scriptProfile =
				{
					order = get_order(),
					name = L["Script Profile"],
					desc = L["CPU profiling is disabled by default since it has some overhead. CPU profiling is controlled by this option."],
					type = "toggle",
					set = function(info,val)
						ControlPanel:SetCVar("scriptProfile",val)
					end,
					get = function(info)
						return ControlPanel:GetCVarBool("scriptProfile")
					end,
				},
			}
		}
	},
}

local optionsFrames = {}
function ControlPanel_Options:OnInitialize()
	options.args.profile = AceDBOptions:GetOptionsTable(ControlPanel.db)
	AceConfig:RegisterOptionsTable("ControlPanel", options, nil)
	optionsFrames.general = AceConfigDialog:AddToBlizOptions("ControlPanel", "ControlPanel")
	ControlPanel.db.RegisterCallback(ControlPanel, "OnProfileChanged", "OnEnable")
	ControlPanel.db.RegisterCallback(ControlPanel, "OnProfileCopied", "OnEnable")
	ControlPanel.db.RegisterCallback(ControlPanel, "OnProfileReset", "OnEnable")
end